@@ -71,11 +71,6 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
    Written by Per Bothner <bothner@cygnus.com>, July 1993.  */
 
 #include "hconfig.h"
-#ifdef __STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include "system.h"
 #include "gansidecl.h"
 #include "obstack.h"
@@ -341,7 +336,7 @@ setgid\0setpgid\0setsid\0setuid\0sleep\0sysconf\0tcgetpgrp\0tcsetpgrp\0\
 ttyname\0unlink\0write\0" },
   { CONTINUED, POSIX2_SYMBOL, "getopt\0" },
   { CONTINUED, XOPEN_EXTENDED_SYMBOL,
-      "lockf\0gethostid\0gethostname\0readlink\0" },
+      "lockf\0gethostid\0gethostname\0readlink\0symlink\0" },
 
   { "utime.h", POSIX1_SYMBOL, "utime\0" },
 
